@@ -1,6 +1,6 @@
 package User;
 
-use Mojo::Base '-base';
+use Mojo::Base -base;
 
 has 'age';
 has 'bio';
@@ -10,7 +10,7 @@ has 'orders';
 
 package TestHelper;
 
-use Mojo::Base '-strict';
+use Mojo::Base -strict;
 use Test::More;
 
 require Exporter;
@@ -20,7 +20,7 @@ our @EXPORT = qw(render_input user dom is_field_count is_field_attrs);
 sub dom { shift->tx->res->dom }	# For Test::Mojo
 
 sub is_field_count
-{ 
+{
     my ($t, $field, $expect) = @_;
     # can't say ->input->size unless input() exists
     is(dom($t)->find($field)->size, $expect, "$field count");
@@ -30,21 +30,29 @@ sub is_field_attrs
 {
     my ($t, $field, $expect) = @_;
     my $e = dom($t)->at($field);
-    my $attrs = $e ? $e->attr : {};
+
+    my $attrs;
+    if($e) {
+	# attrs removed after 4.50
+	$attrs = $e->can('attrs') ? $e->attrs : $e->attr;
+    }
+
+    $attrs //= {};
+
     is_deeply($attrs, $expect, "$field attributes");
 }
 
-sub user 
-{ 
+sub user
+{
     my %attribs = @_;
-    my %defaults = (admin => 1, 
-		    age   => 101,		    
+    my %defaults = (admin => 1,
+		    age   => 101,
 		    bio   => 'Proprietary and confidential',
 		    name  => 'sshaw',
 		    orders => [ { id => 1 }, { id => 2 } ]);
     %attribs = (%defaults, %attribs);
 
-    User->new(%attribs);	
+    User->new(%attribs);
 }
 
 sub render_input
@@ -61,6 +69,3 @@ sub render_input
 }
 
 1;
-
-
-
